/*
 * Copyright (c) 2006 Courtney Cavin, John Gilbert
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <stdlib.h>
#include <string.h>
#include "pz.h"

/* goal #1, make fullscreen: DONE */
/* goal #2, make hires: DONE */
/* goal #3, make super fast (30-60 fps): pending*/
/* 	change loop timing: DONE */
/* 	make draw_noise loop tighter: DONE */
/* 	remove ttk_makecol from loop: DONE */
/* 	remove rand from loop: DONE, not much improvement */
/* 	remove ttk_pixel from loop: DONE */

#ifdef SDL

static PzModule *module;
#define XMAX (ttk_screen->w)
#define YMAX (ttk_screen->h)
static int noise_poolsize;
static short *noise_pool;

static void draw_noise(PzWidget *wid, ttk_surface srf)
{
	/* notice that this draws directly to the screen */
	memcpy(ttk_screen->srf->pixels, (char *)((int)noise_pool + 
				((rand() % (noise_poolsize - (XMAX*YMAX)))&~1)),
			XMAX*YMAX * ttk_screen->srf->format->BytesPerPixel);
	ttk_gfx_update(ttk_screen->srf);
}

#define safe_free(x) do { if (x) free(x); x = 0; } while(0)
static int noisy_events(PzEvent *e)
{
	int ret = TTK_EV_UNUSED;
	if (e->type == PZ_EVENT_BUTTON_DOWN && e->arg == PZ_BUTTON_MENU) {
		pz_close_window(e->wid->win);
		safe_free(noise_pool);
		ret = 0;
	}
	return ret;
}

static int mix_up_the_noise(TWidget *this)
{
	this->dirty = 1;
	return 0;
}

static void enough_noise()
{
	safe_free(noise_pool);
}

static PzWindow *more_noise_please()
{
	int i;
	PzWindow *ret;
	PzWidget *wid;
	ttk_color *colors;

	noise_poolsize = 3 * XMAX * YMAX + 149; /* not an even mult of screen size */
	colors = malloc(256 * sizeof(ttk_color));
	if (!colors) return TTK_MENU_DONOTHING;

	for (i = 0; i <= 255; i++)
		colors[i] = ttk_makecol(i,i,i);

	if ((ttk_screen->srf->format->BytesPerPixel + 1) & ~3) {
		pz_error("Unknown screen type");
		free(colors);
		return TTK_MENU_DONOTHING;
	}

	noise_pool = malloc(noise_poolsize *
			ttk_screen->srf->format->BytesPerPixel);

	if (!noise_pool) {
		free(colors);
		return TTK_MENU_DONOTHING;
	}

	if (ttk_screen->srf->format->BytesPerPixel == 1) {
		for (i = 0; i < noise_poolsize; i++)
			((char *)noise_pool)[i] = colors[(rand() >> 2) & 0xff];
	}
	else if (ttk_screen->srf->format->BytesPerPixel == 2) {
		for (i = 0; i < noise_poolsize; i++)
			noise_pool[i] = colors[(rand() >> 2) & 0xff];
	}
	free(colors);

	/* technically, we don't need this window anymore... */
	ret = pz_new_window(_("Noise"), PZ_WINDOW_NORMAL);
	wid = pz_add_widget(ret, draw_noise, noisy_events);
	ttk_widget_set_timer(wid, 15); /* ms, was 70 */
	wid->timer = mix_up_the_noise;
	/* Make it fullscreen */
	ttk_window_hide_header( ret );
	ret->x = 0;
	ret->y = 0;
	ret->w = ttk_screen->w;
	ret->h = ttk_screen->h;

	return pz_finish_window(ret);
}

static void init_noise()
{
	module = pz_register_module("noise", enough_noise);
	pz_menu_add_action("/Extras/Demos/Noise", more_noise_please);
}

PZ_MOD_INIT(init_noise)

#else
#warning "'noise' only works with a SDL backend"
#endif
